package org.openjdk.gcbench;

import org.openjdk.gcbench.util.Dummy;
import org.openjdk.jmh.runner.Runner;
import org.openjdk.jmh.runner.RunnerException;
import org.openjdk.jmh.runner.options.Options;
import org.openjdk.jmh.runner.options.OptionsBuilder;
import org.openjdk.jmh.runner.options.VerboseMode;

import java.io.PrintWriter;

public class MaxHeapDetector {

    public static int MAX_HEAP;

    public static void override(int value) {
        MAX_HEAP = value;
    }

    public static void init(PrintWriter pw) {
        int baseHeapMB = 1000;
        int latestSuccessMB = 0;
        boolean progress;
        do {
            progress = false;
            for (int incr = 1000; incr < Integer.MAX_VALUE; incr *= 2) {
                int heapGB = baseHeapMB + incr;
                pw.print(heapGB + "? ");
                pw.flush();
                Options opts = new OptionsBuilder()
                        .include(Dummy.class.getCanonicalName())
                        .threads(1)
                        .jvmArgsAppend("-Xmx" + heapGB + "m", "-Xms" + heapGB + "m", "-XX:+AlwaysPreTouch")
                        .verbosity(VerboseMode.SILENT)
                        .build();
                try {
                    new Runner(opts).runSingle();
                    latestSuccessMB = heapGB;
                    progress = true;
                } catch (RunnerException e) {
                    baseHeapMB = latestSuccessMB;
                    break;
                }
            }
        } while (progress);

        pw.println();
        pw.println("Max heap size is " + latestSuccessMB + " Mb");
        pw.println();

        MAX_HEAP = latestSuccessMB;
    }

}
