package org.openjdk.gcbench.runtime.writes;

import org.openjdk.jmh.annotations.*;

import java.util.concurrent.TimeUnit;

import static org.openjdk.gcbench.util.Sinks.sink;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(1)
@State(Scope.Benchmark)
public class Volatile {

    Target target;

    @Setup
    public void setup() {
        target = new Target();
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_none() {
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_boolean() {
        target.f_boolean = true;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_byte() {
        target.f_byte = 42;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_char() {
        target.f_char = 'A';
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_short() {
        target.f_short = 42;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_int() {
        target.f_int = 42;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_float() {
        target.f_float = 42f;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_long() {
        target.f_long = 42L;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_double() {
        target.f_double = 42D;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test_Object() {
        target.f_Object = target;
    }

    static class Target {
        volatile boolean f_boolean;
        volatile byte f_byte;
        volatile short f_short;
        volatile char f_char;
        volatile int f_int;
        volatile float f_float;
        volatile long f_long;
        volatile double f_double;
        volatile Object f_Object;
    }

}
