package org.openjdk.gcbench.runtime.writes;

import org.openjdk.jmh.annotations.*;

import java.util.concurrent.TimeUnit;

@Warmup(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Measurement(iterations = 5, time = 1, timeUnit = TimeUnit.SECONDS)
@Fork(1)
@BenchmarkMode(Mode.AverageTime)
@OutputTimeUnit(TimeUnit.NANOSECONDS)
@Threads(1)
@State(Scope.Thread)
public class WriteBarriersPrimitive {

    Target target;
    int source;

    @Setup
    public void setup() {
        target = new Target();
        source = 42;
    }

    @Benchmark
    @CompilerControl(CompilerControl.Mode.DONT_INLINE)
    public void test() {
        target.field = source;
    }

    static class Target {
        int field;
    }

    /*
       i5 4210U, 1.7 Ghz, Linux x86_64, JDK 9 (Shenandoah, 2016-09-12)

           Benchmark                    Mode  Cnt  Score   Error  Units

           # Shenandoah
           WriteBarriersPrimitive.test  avgt   25  3.771 ± 0.010  ns/op

           # G1
           WriteBarriersPrimitive.test  avgt   25  2.623 ± 0.022  ns/op

           # Parallel
           WriteBarriersPrimitive.test  avgt   25  2.551 ± 0.008  ns/op

        Shenandoah has to do the write barriers even for primitive stores,
        in order to preserve "no writes into evacuated regions" invariant.
        Other collectors do not have to do this. The write barrier code
        quality may be improved to amortize the costs.

        Shenandoah:

                            [Verified Entry Point]
          6.81%    8.72%      0x00007f85a58dddc0: mov    %eax,-0x14000(%rsp)
          3.08%    3.37%      0x00007f85a58dddc7: push   %rbp
          0.05%    0.11%      0x00007f85a58dddc8: sub    $0x10,%rsp
          8.72%    7.79%      0x00007f85a58dddcc: mov    -0x8(%rsi),%r10        ; <--- read barrier ($this)
          0.67%    0.50%      0x00007f85a58dddd0: mov    0x10(%r10),%r11d       ; get field $target
          0.08%    0.09%      0x00007f85a58dddd4: mov    0xc(%r10),%r10d        ; get field $source
          6.83%    7.37%      0x00007f85a58dddd8: test   %r11d,%r11d            ; null check $target
                          ╭   0x00007f85a58ddddb: je     0x00007f85a58dde0e
          2.30%    0.64%  │   0x00007f85a58ddddd: shl    $0x3,%r11              ; unpack $target
          0.61%    0.52%  │   0x00007f85a58ddde1: mov    -0x8(%r11),%r11        ; <--- read barrier once; WHY? There is a null check before already
          0.14%    0.09%  │   0x00007f85a58ddde5: cmpb   $0x0,0x3d8(%r15)       ; evacuation in progress?
          6.70%    6.03%  │   0x00007f85a58ddded: mov    -0x8(%r11),%r11        ; <--- read barrier twice (in sequence now)
          3.53%    1.67%  │╭  0x00007f85a58dddf1: je     0x00007f85a58dddfe
                          ││  0x00007f85a58dddf3: xchg   %rax,%r11              ; <--- barrier slowpath
                          ││  0x00007f85a58dddf6: callq  Stub::shenandoah_wb
                          ││  0x00007f85a58dddfb: xchg   %rax,%r11
          0.53%    0.62%  │↘  0x00007f85a58dddfe: mov    %r10d,0xc(%r11)        ; actual store
         25.37%   27.26%  │   0x00007f85a58dde02: add    $0x10,%rsp
          2.28%    2.20%  │   0x00007f85a58dde06: pop    %rbp
          0.72%    0.03%  │   0x00007f85a58dde07: test   %eax,0x11df41f3(%rip)
          0.06%    0.09%  │   0x00007f85a58dde0d: retq


        Parallel:

                          [Verified Entry Point]
          4.35%    5.11%    0x00007fc6fdaf3840: mov    %eax,-0x14000(%rsp)
         10.05%    9.23%    0x00007fc6fdaf3847: push   %rbp
          0.05%             0x00007fc6fdaf3848: sub    $0x10,%rsp
          4.65%    4.18%    0x00007fc6fdaf384c: mov    0xc(%rsi),%r11d          ; get field $source
         10.83%    9.16%    0x00007fc6fdaf3850: mov    0x10(%rsi),%r10d         ; get field $target
          0.02%    0.02%    0x00007fc6fdaf3854: mov    %r11d,0xc(%r12,%r10,8)   ; actual field store
         14.61%   15.76%    0x00007fc6fdaf3859: add    $0x10,%rsp
                            0x00007fc6fdaf385d: pop    %rbp
          6.13%    6.24%    0x00007fc6fdaf385e: test   %eax,0x13c1a79c(%rip)
          0.02%             0x00007fc6fdaf3864: retq

     */

}
